﻿using System;

namespace AutoFixture.AutoNSubstitute
{
    /// <summary>
    /// An attribute that can be applied to code elements to indicate they should be auto-generated by creating a
    /// substitute.
    /// </summary>
    /// <remarks>
    /// When this attribute is applied to a parameter in a data-driven test, such as xUnit's <c>[Theory]</c>,
    /// the test must also have a test conventions attribute that enables automatic generation of the
    /// <see cref="Fixture"/> with the <see cref="AutoNSubstituteCustomization"/>. For xUnit, this is done by deriving
    /// the test conventions attribute from the <c>AutoDataAttribute</c> supplied by the AutoFixture for xUnit.net.
    /// </remarks>
    [AttributeUsage(
        AttributeTargets.Parameter | AttributeTargets.Property | AttributeTargets.Field,
        AllowMultiple = false)]
    public sealed class SubstituteAttribute : Attribute
    {
    }
}
